<?php
// Define the core domain concepts

class Product
{
    private $name;
    private $description;
    private $price;
    private $quantity;

    public function __construct($name, $description, $price, $quantity)
    {
        $this->name = $name;
        $this->description = $description;
        $this->price = $price;
        $this->quantity = $quantity;
    }

    public function getName()
    {
        return $this->name;
    }

    public function getDescription()
    {
        return $this->description;
    }

    public function getPrice()
    {
        return $this->price;
    }

    public function getQuantity()
    {
        return $this->quantity;
    }

    public function decrementQuantity($amount)
    {
        $this->quantity -= $amount;
    }
}

class Customer
{
    private $name;
    private $orders = [];

    public function __construct($name)
    {
        $this->name = $name;
    }

    public function getName()
    {
        return $this->name;
    }

    public function placeOrder($product, $quantity, $total)
    {
        $this->orders[] = new Order($product, $quantity, $total);
    }

    public function getOrders()
    {
        return $this->orders;
    }
}

class Order
{
    private $product;
    private $quantity;
    private $total;

    public function __construct($product, $quantity, $total)
    {
        $this->product = $product;
        $this->quantity = $quantity;
        $this->total = $total;
    }

    public function getProduct()
    {
        return $this->product;
    }

    public function getQuantity()
    {
        return $this->quantity;
    }

    public function getTotal()
    {
        return $this->total;
    }
}


// Define the repositories

class ProductRepository
{
    private $products = [];

    public function addProduct($product)
    {
        $this->products[$product->getName()] = $product;
    }

    public function removeProduct($product)
    {
        unset($this->products[$product->getName()]);
    }

    public function getProductByName($name)
    {
        return isset($this->products[$name]) ? $this->products[$name] : null;
    }
}

class CustomerRepository
{
    private $customers = [];

    public function addCustomer($customer)
    {
        $this->customers[$customer->getName()] = $customer;
    }

    public function removeCustomer($customer)
    {
        unset($this->customers[$customer->getName()]);
    }

    public function getCustomerByName($name)
    {
        return isset($this->customers[$name]) ? $this->customers[$name] : null;
    }
}


// Define the services

class OrderService
{
    private $productRepository;
    private $customerRepository;

    public function __construct($productRepository, $customerRepository)
    {
        $this->productRepository = $productRepository;
        $this->customerRepository = $customerRepository;
    }

    public function placeOrder($customerName, $productName, $quantity)
    {
        $customer = $this->customerRepository->getCustomerByName($customerName);
        $product = $this->productRepository->getProductByName($productName);

        if (!$customer) {
            throw new Exception("Customer with name '{$customerName}' not found");
        }

        if (!$product) {
            throw new Exception("Product with name '{$productName}' not found");
		}

		if ($product->getQuantity() < $quantity) {
			throw new Exception("Insufficient quantity of product '{$productName}'");
		}

		$total = $product->getPrice() * $quantity;

		$customer->placeOrder($product, $quantity, $total);
		$product->decrementQuantity($quantity);

		$this->customerRepository->addCustomer($customer);
		$this->productRepository->addProduct($product);
	}
}

// Define the console application

$productRepository = new ProductRepository();
$productRepository->addProduct(new Product('T-Shirt', 'A simple cotton T-Shirt', 20, 100));
$productRepository->addProduct(new Product('Socks', 'Cotton Socks', 10, 10));

$customerRepository = new CustomerRepository();
$customerRepository->addCustomer(new Customer('Laza'));

$orderService = new OrderService($productRepository, $customerRepository);

echo "Welcome to the e-commerce application!\n\n";

$customerName = readline("Please enter your name: ");
$productName = readline("Please enter the product name: ");
$quantity = readline("Please enter the quantity: ");

try {
	$orderService->placeOrder($customerName, $productName, $quantity);
	echo "\nOrder successfully placed!\n";
} catch (Exception $e) {
	echo "\nError: " . $e->getMessage() . "\n";
}
